<?php

namespace MakeCommerce\Payment;

/**
 * Creates minimal functionality needed for payment gateways
 * 
 * @since 3.0.0
 */

abstract class Gateway extends \WC_Payment_Gateway {

    use Gateway\Refund, Gateway\Subscription;

    public $id;
    public $MK;

    /**
     * Defines which WooCommerce options this payment gateway supports
     */
    public $supports = array(
        'subscriptions',
        'subscription_cancellation', 
        'subscription_suspension', 
        'subscription_reactivation',
        'subscription_amount_changes',
        'subscription_date_changes',
        'subscription_payment_method_change',
        'products',
        'refunds'
    );
    
    /**
     * Construct payment gateway
     * 
     * @since 3.0.0
     */
    public function __construct() {

        //exit if the gateway is not enabled
        if ( !$this->enabled() ) {
            return;
        }

        //Set api
        $this->MK = \MakeCommerce::get_api();

        //get settings from \WC_Payment_Gateway
        $this->init_settings();

        //set default settings used by all shipping methods
        $this->set_default_settings();

        //initialize form fields for admin
        $this->initialize_form_fields();

        //set hooks
        $this->set_hooks();

        //set gateway specific hooks
        if ( $this->active() ) {
            $this->set_gateway_hooks();
        }
    }

    /**
     * Initializes basic fields used by all payment gateways.
     * 
     * @since 3.0.0
     */
    public function initialize_form_fields() {

        //Initialize basic form fields
        $this->initialize_basic_form_fields();

        //Initialize gateway specific form fields
        $this->initialize_gateway_type_form_fields();

    }

    /**
     * Check if the gateway is available
     * Automatically called by \WC_Payment_Gateway
     * 
     * @since 3.0.0
     */
    public function is_available() {

        if ( isset($this->settings['active']) ) {
            if ( !( $this->settings['active'] == "yes" && \MakeCommerce::is_api_set() ) ) {
                return false;
            }
        }

        return parent::is_available();
    }

    /**
     * Sets default settings used by all payment gateways
     * 
     * @since 3.0.0
     */
    final private function set_default_settings() {

        if ( isset( $this->settings['active'] ) ) {
            $this->enabled = $this->settings['active'];
        }
    }

    /**
     * Checks whether the payment gateway is marked as active
     * 
     * @since 3.0.0
     */
    final public function active() {

        if ( isset( $this->settings['active'] ) ) {
            if ( $this->settings['active'] == "yes" || empty( $this->settings['active'] ) ) {
                return true;
            }
        }

        return false;
    }

    /**
     * Set hooks used by all WooCommerce payment gateways
     * 
     * @since 3.0.0
     */
    final private function set_hooks() {
        
        add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options' ) );
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        add_action( 'woocommerce_payment_gateways', array( $this, 'add_payment_gateway' ) );

        global $woocommerce;

		if ( ( double )$woocommerce->version < 2.6 ) {
			add_action( 'admin_notices', 'old_woocommerce_notice' );
		}
		
		//dont know if this is needed:
		// Woocommerce Multilingual overrides
		if ( \MakeCommerce\i18n::using_language_plugins() ) {
			add_filter( 'get_post_metadata', array( $this, 'override_payment_method_string' ), 5, 4 );
		}

		//add slice button to product page
		add_action( 'woocommerce_after_add_to_cart_button', array( $this, 'product_slice_button' ), 20 );
    }

    /**
	 * Overrides payment method title
	 * 
	 * @since 3.0.0
	 */
	public function override_payment_method_string( $check, $object_id, $meta_key, $single ) {
        
		if ( $meta_key == '_payment_method_title' ) {

			$payment_method = get_post_meta( $object_id, '_payment_method', true );
			
			if ( $payment_method == 'makecommerce' ) {
                $payment_gateways = WC()->payment_gateways->payment_gateways();

				if ( isset( $payment_gateways[$payment_method] ) ) {
					$title = $payment_gateways[$payment_method]->title;
			
					return $title;
				}
			}
		}

		return $check;
    }
    
    /**
	 * Show slice button on product view after add to cart button
	 * 
	 * @since    3.0.0
	 */
	public function product_slice_button() {

        if ( get_option('mk_product_slice', 'no') == 'yes' ) { 

			global $product;
            global $wpdb;
            
            //ignore slice for subscription products
            if ( class_exists( '\WC_Subscriptions_Product' ) && \WC_Subscriptions_Product::is_subscription( $product ) ) {
                return;
            }

			$price_incl_tax = wc_get_price_including_tax( $product );
			
			$methods = $wpdb->get_results('SELECT * FROM ' . $wpdb->prefix . MAKECOMMERCE_TABLENAME);
			$slice = false;

			if ( count($methods) ) {
				foreach ( $methods as $method ) {
					if ( $method->name =="slice" ) {
						if ( $price_incl_tax >= $method->min_amount ) {
							if ( $price_incl_tax <= $method->max_amount ) {
								$slice = true;
							}
						}
					}
				}
			}

			if ( $slice == true ) {
				echo '<div style="width:100%; margin-top: 20px; margin-bottom: auto;"><img src="'.plugins_url('woocommerce/images/slice_logo_dark_176x62.svg', __FILE__).'" width="88" weight="31"/>&nbsp;&nbsp;'.__("Buy now, pay later in 3 instalments", "wc_makecommerce_domain").' 3 x '.sprintf("%.2f",$price_incl_tax/3).' €</div>';
			}   
		}
	}

    /**
     * Adds payment gateway to woocommerce
     * 
     * @since 3.0.0
     */
    final public function add_payment_gateway( $methods ) {

        $methods[] = get_class($this);

		return $methods;
    }

    /**
     * Initializes basic shared form fields
     * Used by all payment gateways
     * 
     * since 3.0.0
     */
    final private function initialize_basic_form_fields() {

        $this->form_fields = array();
        
        $this->form_fields['logo'] = array( 'type' => 'title', 'description' => \MakeCommerce::get_logo_html() );	
    }

    /**
     * Initializes gateway type form fields
     * 
     * @since 3.0.0
     */
    abstract public function initialize_gateway_type_form_fields();

    /**
     * Force creating a function for setting gateway specific hooks
     * 
     * @since 3.0.0
     */
    abstract public function set_gateway_hooks();

    /**
     * Checks if the gateway is enabled
     * 
     * @since 3.0.0
     */
    abstract public function enabled();
}