<?php

namespace MakeCommerce\Shipping;

/**
 * All functionality that has to do with shipping labels and printing
 * 
 * @since 3.0.0
 */

class Label extends \MakeCommerce\Shipping {

	private $loader;
	
	//page format used for creating labels
	public $page_fromat = "A4";

	/**
	 * Constructs Label class, defines loader and hooks
	 * 
	 * @since 3.0.0
	 */
    public function __construct( \MakeCommerce\Loader $loader ) {
    
		$this->loader = $loader;
		
		$this->define_hooks();
	}
	
	/**
	 * Define all wordpress hooks needed for printing stuff and creating labels
	 * 
	 * @since 3.0.0
	 */
    public function define_hooks() {

        $this->loader->add_action( 'woocommerce_order_actions_end', $this, 'print_button' );
		$this->loader->add_action( 'wp_ajax_print_pml', $this, 'print' );
		$this->loader->add_filter( 'admin_action_parcel_machine_print_labels', $this, 'bulk_print' );
		$this->loader->add_filter( 'admin_footer', $this, 'bulk_print_and_register' );
		$this->loader->add_filter( 'admin_action_parcel_machine_labels', $this, 'bulk_register' );
    }

	/**
	 * Creates shipping label using MakeCommerce API and redirect to the page where it is available or return data
	 * 
	 * @since 3.0.0
	 */
    public function create_labels( $post_ids, $ajax = false ) {

		if ( !is_array( $post_ids ) ) {
			$post_ids = array( $post_ids );
		}
        
        $this->initalize();
        
		$shipping_request = array( 'credentials' => array(), 'orders' => array(), 'printFormat' => $this->page_fromat );
		
		foreach ( $this->shipping_methods as $shipping_method ) {
			$shipping_classes_map[$shipping_method["method"]] = $shipping_method["class"];
		}
        
        $missing_ids = array();
        
		foreach ( $post_ids as $post_id ) {

            $shipment_id = get_post_meta( $post_id, '_parcel_machine_shipment_id', true );
            
			if ( !$shipment_id ) { 
				$missing_ids[] = $post_id;
				continue;
            }
            
			$order = new \WC_Order( $post_id );
            $shipping_methods = $order->get_shipping_methods();
            
            if ( empty( $shipping_methods ) ) {
				continue;
			}
            
			foreach ( $shipping_methods as $shipping_method ) {

				$shipping_id = explode( ':', $shipping_method['method_id'] );
				$shipping_class = $shipping_id[0];
                $shipping_instance = !empty( $shipping_id[1] ) ? $shipping_id[1] : null;
                
				if ( empty( $shipping_classes_map[$shipping_class] ) ) {
					continue;
				}

				$transport_class = new $shipping_classes_map[$shipping_class]($shipping_instance);
				$carrier_uc = mb_strtoupper( $transport_class->carrier );
				
				//apt == parcelmachine
				if ( $transport_class->type === 'apt' ) {

					$parcel_machine = get_post_meta( $post_id, '_parcel_machine', true );

					if ( !$parcel_machine ) {
						continue;
					}

					list( $carrier, $machine_id ) = explode( '||', $parcel_machine );

					if ( !$carrier || !$machine_id ) {
						continue;
					}
                }
                
				if ( empty( $shipping_request['credentials'][$carrier_uc] ) ) {

					$api_user = $transport_class->settings['service_user'];
					$api_password = $transport_class->settings['service_password'];

					$use_mk_contract = false;

					if ( isset( $transport_class->settings['use_mk_contract'] ) ) {
						$use_mk_contract = $transport_class->settings['use_mk_contract'];
					}

					//change carrier_uc if it has been set in the interface.
					if ( isset( $transport_class->settings['service_carrier'] ) ) {
						$carrier_uc = $transport_class->settings['service_carrier'];
					}

					if ( ( !$api_user || !$api_password ) and ( !$use_mk_contract ) ) {

						$this->loader->add_action( 'admin_notices', $this, 'mk_admin_error' );
						continue;
					}

					if ( !$use_mk_contract ) {
						$shipping_request['credentials'][$carrier_uc] = array( 'carrier' => $carrier_uc, 'username' => $api_user, 'password' => $api_password );
					}
                }
                
				$sender = array(
					'name' => $transport_class->settings['shop_name'],
					'phone' => $transport_class->settings['shop_phone'],
					'email' => $transport_class->settings['shop_email'],
					'postalCode' => !empty( $transport_class->settings['shop_postal_code'] ) ? $transport_class->settings['shop_postal_code'] : '',
					'country' => !empty( $transport_class->settings['shop_address_country'] ) ? $transport_class->settings['shop_address_country'] : '',
					'city' => !empty( $transport_class->settings['shop_address_city'] ) ? $transport_class->settings['shop_address_city'] : '',
					'street' => !empty( $transport_class->settings['shop_address_street'] ) ? $transport_class->settings['shop_address_street'] : ''
                );
                
				$s_first_name = method_exists( $order, 'get_shipping_first_name' ) ? $order->get_shipping_first_name() : $order->shipping_first_name;
				$s_last_name = method_exists( $order, 'get_shipping_last_name' ) ? $order->get_shipping_last_name() : $order->shipping_last_name;
                $recipient = trim( $s_first_name . ' ' . $s_last_name );
                
				if ( empty( $recipient ) ) {

					$b_first_name = method_exists( $order, 'get_billing_first_name' ) ? $order->get_billing_first_name() : $order->billing_first_name;
					$b_last_name = method_exists( $order, 'get_billing_last_name' ) ? $order->get_billing_last_name() : $order->billing_last_name;
					$recipient = trim( $b_first_name . ' ' . $b_last_name );
                }
                
				$b_phone = method_exists( $order, 'get_billing_phone' ) ? $order->get_billing_phone() : $order->billing_phone;
				$b_email = method_exists( $order, 'get_billing_email' ) ? $order->get_billing_email() : $order->billing_email;

				$sr_order = array(
					'carrier' => $carrier_uc,
					'orderId' => $order->get_id(),
					'recipient' => array( 'name' => $recipient, 'phone' => $b_phone, 'email' => $b_email ),
					'sender' => $sender,
					'shipmentId' => $shipment_id
                );
                
				$m_service_type = null;

				if ( $transport_class->carrier === 'omniva' && $transport_class->type === 'atp' ) {
					$m_service_type = 'PA';
				} else if ( !empty( $transport_class->settings['registerOnPaymentCode'] ) ) {
					$m_service_type = $transport_class->settings['registerOnPaymentCode'];
				}

				if ( !empty( $machine_id ) ) {
					$sr_order['destination']['destinationId'] = $machine_id;
                }
                
				if ( $transport_class->type === 'cou' ) {

					$sr_order['destination'] = array(
						'postalCode' => $order->shipping_postcode,
						'country' => $order->shipping_country,
						'county' => $order->shipping_state,
						'city' => $order->shipping_city,
						'street' => $order->shipping_address_1 . ' ' . $order->shipping_address_2
					);

					if ( $shipping_class == "courier_smartpost" ) {
						
						$delivery_time = get_post_meta( $post_id, '_delivery_time', true );
						
						if ( $delivery_time ) {
							$sr_order['destination']['timeWindow'] = $delivery_time;
						}
					}
				}

				if ( $m_service_type ) {
					$sr_order['services'] = array( 'serviceType' => $m_service_type );
				}

				$shipping_request['orders'][] = $sr_order;
			}
        }
        
        $shipping_request['credentials'] = array_values( $shipping_request['credentials'] );
        
		if ( empty( $shipping_request['orders'] ) ) {
			return;
        }
        
        $MK = \MakeCommerce::get_api();
        
		if ( !$MK ) {
			return;
        }
        
		try {
			$response = $MK->createLabels( $shipping_request );
		} catch ( \Exception $e ) {

			echo $e->getMessage();
			exit();
        }
        
		if ( empty( $response->labelUrl ) ) {
			return;
        }
        
		if ( $ajax ) {
			return $response->labelUrl;
        }
        
		$mk_err = false;
		if ( !empty( $missing_ids ) ) {
            $mk_err = sprintf( __( 'Orders %s did not have shipment ID attached so no labels was printed! Please register those packages if you think this is an error!', 'wc_makecommerce_domain' ), join( ', ', $missing_ids ) );
        }

		$sendback = add_query_arg( array( 'post_type' => 'shop_order', 'mk_pdf' => urlencode( $response->labelUrl ), 'mk_err' => urlencode( $mk_err ) ), '' );
        
        wp_redirect( esc_url_raw( $sendback ) );
        
        exit();
	}

	/**
	 * Prints labels using ajax function in admin
	 * 
	 * @since 3.0.0
	 */
    public function print() {

		if ( !current_user_can( 'manage_woocommerce' ) ) {
			die();
        }
        
        echo $this->create_labels( array( intval( $_POST['id'] ) ), true );
        
		die();
	}
	
	/**
	 * Prints labels for all selected orders
	 * 
	 * @since 3.0.0
	 */
	public function bulk_print() {

		$wp_list_table = _get_list_table('WP_Posts_List_Table');
		$post_ids = array_map('absint', (array)$_REQUEST['post']);

		$this->create_labels( $post_ids );
	}

	/**
	 * Adds bulk shipment register and printing actions to order view
	 * 
	 * @since 3.0.0
	 */
	public function bulk_print_and_register() {

		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-datepicker' );
		wp_enqueue_script( 'wc_mk_timepicker', plugins_url( '/js/jquery-ui.timepicker.js', __FILE__ ) );

		wp_enqueue_style( 'wc_mk_timepicker-css', plugins_url( '/css/jquery-ui.timepicker.css', __FILE__ ) );

		global $post_type;
		
		if ( 'shop_order' === $post_type ) { ?>
            <script type="text/javascript">
            jQuery(function() {
                jQuery('<option>').val('parcel_machine_labels').text('<?php _e( 'Register parcel machine shipments', 'wc_makecommerce_domain' )?>').appendTo('select[name="action"]');
                jQuery('<option>').val('parcel_machine_labels').text('<?php _e( 'Register parcel machine shipments', 'wc_makecommerce_domain' )?>').appendTo('select[name="action2"]');
                jQuery('<option>').val('parcel_machine_print_labels').text('<?php _e( 'Print parcel machine labels', 'wc_makecommerce_domain' )?>').appendTo('select[name="action"]');
                jQuery('<option>').val('parcel_machine_print_labels').text('<?php _e( 'Print parcel machine labels', 'wc_makecommerce_domain' )?>').appendTo('select[name="action2"]');
				jQuery('select[name="action"]').after('<input type="text" name="transport_time" class="hidden timepicker"/>');
				jQuery('select[name="action"], select[name="action2"]').on('change', function(){
					var val = jQuery(this).val();
					if (val === 'parcel_machine_order_courier') {
						jQuery('.timepicker').removeClass('hidden');
					} else {
						jQuery('.timepicker').addClass('hidden');
					}
				});
				jQuery('.tablenav.bottom .actions.bulkactions').hide();
				var now = new Date();
				var mins = now.getMinutes();
				var quarterHours = Math.ceil(mins/15);
				if (quarterHours == 4) { now.setHours(now.getHours()+1); }
				var rounded = (quarterHours*15)%60;
				now.setMinutes(rounded);
				jQuery('.timepicker').datetimepicker({dateFormat: 'dd/mm/yy', timeFormat: 'HH:mm', stepMinute: 15, minDateTime: now, devaultValue: jQuery.datepicker.formatTime('dd/mm/yyyy HH:mm', now)});
			});
			</script>
			<?php if (!empty($_REQUEST['mk_err'])) { ?>
				<script type="text/javascript">
				jQuery(function(){
					alert('<?php echo htmlspecialchars($_REQUEST['mk_err']) ?>');
				});
				</script>
			<?php } ?>
			<?php if (!empty($_REQUEST['mk_pdf'])) { ?>
				<script type="text/javascript">
				jQuery(function(){
					window.open('<?php echo $_REQUEST['mk_pdf'] ?>', 'pdf');
				});
				</script>
			<?php }
		}
    }

	/**
	 * Registers shipments in bulk (admin action)
	 * 
	 * @since 3.0.0
	 */
	public function bulk_register() {

		$wp_list_table = _get_list_table( 'WP_Posts_List_Table' );
		$post_ids = array_map( 'absint', ( array )$_REQUEST['post'] );

		$this->register_shipment( $post_ids );
	}

	/**
	 * Supporting function for print_labels. Creates needed javasript for printing button
	 * 
	 * @since 3.0.0
	 */
    public function print_button( $post_id ) {

        $shipment_id = get_post_meta( $post_id, '_parcel_machine_shipment_id', true );
        
        if ( !$shipment_id ) {
            return;
        }
        
	    ?>
	    <li class="wide">
		    <a id="print_parcel_machine_label" href="#" class="button"><?php _e( 'Print parcel label', 'wc_makecommerce_domain' );?></a>
		    <img class="mc_loading" src="<?php echo site_url('/wp-admin/images/loading.gif');?>">
			<script type="text/javascript">
				jQuery(function($){
					var data = {
						'action': 'print_pml',
						'id': '<?php echo $post_id; ?>'
					};
					var loading = $('.mc_loading');
					$('#print_parcel_machine_label').click(function(e){
						e.preventDefault();
						var button = $(this);
						button.addClass('disabled');
						loading.show();
						$.post(ajaxurl, data, function(response) {
							button.removeClass('disabled');
							loading.hide();
							window.open(response, 'pdf');
						});
					});
				});
			</script>
		</li>
 		<?php
    }
}