<?php

namespace MakeCommerce\Shipping\Method;

/**
 * Smartpost parcelmachine shipping method.
 * Defines all Smartpost specific options.
 * 
 * @since 3.0.0
 */

abstract class ParcelMachine extends \MakeCommerce\Shipping\Method {
   
    public $type = "apt";
    public $identifier = "parcelmachine";

    /**
     * Required function for all methods, initializes method type
     * 
     * @since 3.0.0
     */
    public function initialize() {

        $this->check_updates();

        //load all hooks/filters for method type
        $this->set_method_type_hooks();

        //set parcelmachine specific properties
        $this->prioritization = null;
        if ( isset( $this->settings['prioritization'] ) ) {
            $this->prioritization = $this->settings['prioritization'];
        }

        $this->short_office_names = null;
        if ( isset( $this->settings['short_office_names'] ) ) {
            $this->short_office_names = $this->settings['short_office_names'];
        }

        //needed for identifier translation
        $parcelmachineString = __( "parcelmachine", 'wc_makecommerce_domain' );
    }

    /**
     * Get method title
     * 
     * @since 3.0.0
     */
    public function return_method_title() {

        return ucfirst( $this->carrier ) . " " . __( "parcel machine (MC)", 'wc_makecommerce_domain' );
    }

    /**
     * Initializes method type specific form fields
     * Loads method specific fields 
     * 
     * @since 3.0.0
     */
    public function initialize_method_type_form_fields() {

        //Initialize method specific fields
        $this->initialize_method_form_fields();

        $this->form_fields['service_user'] = array(
            'title'            => sprintf( __( '%s web services username', 'wc_makecommerce_domain' ), $this->service_name ),
            'type'             => 'text',
            'default'          => ''
        );
        
        $this->form_fields['service_password'] = array(
            'title'            => sprintf( __( '%s web services password', 'wc_makecommerce_domain' ), $this->service_name ),
            'type'             => 'text',
            'default'          => ''
        );

        $this->form_fields['shipping_ui_javascript'] = array(
            'type' => 'shipping_ui_javascript',
        );
    }

    /**
     * Initialize method type specific checkout fields
     * 
     * @since 3.0.0
     */
    public function initialize_method_type_checkout_fields() {

        $this->form_fields['prioritization'] = array(
            'title'            => __('Prioritize', 'wc_makecommerce_domain'),
            'type'             => 'checkbox',
            'label'            => __('Bigger cities will be on top of list, others sorted alphabetically', 'wc_makecommerce_domain'),
            'default'          => 'yes'
        );
        
        $this->form_fields['short_office_names'] = array(
            'title'            => __('Short names', 'wc_makecommerce_domain'),
            'type'             => 'checkbox',
            'label'            => __('Display only parcel machine names, without addresses', 'wc_makecommerce_domain'),
            'default'          => 'no'
        );
    }

    /**
     * Set method type specific hooks/filters
     * 
     * @since 3.0.0
     */
    public function set_method_type_hooks() {

        //add parcelmachines selectbox to checkout
        add_filter( 'woocommerce_review_order_after_shipping' , array( $this, 'add_parcelmachine_checkout_fields' ) );

        //check if a parcelmachine has been chosen in checkout
        add_action( 'woocommerce_checkout_process', array( $this, 'check_parcelmachine_checkout_fields' ) );
        add_action( 'woocommerce_after_checkout_validation', array( $this, 'check_parcelmachine_checkout_fields' ) );
        
        //add shipping metadata to order
        add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'add_parcelmachine_order_meta' ) );
    }

    /**
     * Set parcelmachine specific checkout fields
     * 
     * @since  3.0.0
     */
    public function add_parcelmachine_checkout_fields( $fragments ) {

        $this->order_country = WC()->customer->get_shipping_country();

        //get all parcelmachines
        $machines = \Makecommerce\Shipping::mk_get_machines($this->ext, $this->order_country);

        //sort machines
        $machines = $this->sort_machines( $machines );

        //group machines by city and set name
        $city_machines = array();
        foreach ( $machines as $machine ) {
            if ( $this->short_office_names !== 'yes' ) {
                $machine["name"] .= ' - ' . $machine['city'] . ', ' . $machine['address'];
            }

            $city_machines[$machine["city"]][] = $machine;
        }

        $html = '
        <tr style="display: none;" class="parcel_machine_checkout parcel_machine_checkout_parcelmachine_' . mb_strtolower( $this->ext ) . '">
            <td colspan="2">
                <p class="form-row" id="' . esc_attr( $this->id ) . '_field">
                    <select class="select parcel-machine-select-box" name="' . esc_attr( $this->id ) . '" id="' . esc_attr( $this->id ) . '">
                        <option value="">' . __( '-- select parcel machine --', 'wc_makecommerce_domain' ).'</option>
        ';

        foreach ( $city_machines as $city=>$grouped_machines ) {
            $html .= '<optgroup label="' . $city . '">';

            foreach ( $grouped_machines as $machine ) {
                $html .= '<option value="' . esc_attr( $machine['carrier'] . '||' . $machine['id'] ) . '">' . $machine["name"] . '</option>';
            }

            $html .= '</optgroup>';
        }

        $html .= '
                    </select>
                </p>
            </td>
        </tr>
        ';

        echo $html;
    }

    /**
     * Add check for parcelmachine selectbox
     * Checks if something is chosen
     * 
     * @since 3.0.0
     */
    public function check_parcelmachine_checkout_fields() {

        //those checks are needed, otherwise the notification will be added more than once
        static $added1 = false;
        static $added2 = false;

        $shipping_method = !empty( $_POST['shipping_method'] ) ? $_POST['shipping_method'] : false;

        if ( !empty( $shipping_method[0] ) ) {
            $shipping_method_ext = explode( ':', $shipping_method[0] ); 
        }

        if ( !$added1 && $shipping_method_ext[0] === $this->id && empty ($_POST[$shipping_method_ext[0]] ) ) {
            wc_add_notice( __( '<strong>Parcel machine</strong> is a required field.', 'wc_makecommerce_domain' ), 'error' );
            $added1 = true;
        }
        if ( !$added2 && $shipping_method_ext[0] === $this->id && !$this->valid_phone_number( $_POST['billing_phone'] ) ) {
            wc_add_notice( __( '<strong>Parcel machine</strong> can be used with local phone number only. Please specify your phone number with international access code (like +372xxxxxxx)', 'wc_makecommerce_domain' ), 'error' );
            $added2 = true;
        }
    }

    /**
     * Adds metadata to order about shipping
     * 
     * @since 3.0.0
     */
    public function add_parcelmachine_order_meta( $order_id ) {

        $shipping_method = !empty( $_POST['shipping_method'] ) ? $_POST['shipping_method'] : false;

        if ( !empty($shipping_method[0] ) ) {
            $shipping_method_ext = explode( ':', $shipping_method[0] );
        }

        if ( $shipping_method_ext[0] === $this->id && !empty( $_POST[$this->id] ) ) {

            list( $carrier, $machine ) = explode( '||', $_POST[$this->id] );

            $machine = \MakeCommerce\Shipping::mk_get_machine( $carrier, $machine );
            
            if ( !empty( $machine['id'] ) ) {

                update_post_meta( $order_id, '_shipping_first_name', get_post_meta( $order_id, '_billing_first_name', true ) );
                update_post_meta( $order_id, '_shipping_last_name', get_post_meta( $order_id, '_billing_last_name', true ) );
                update_post_meta( $order_id, '_shipping_address_1', sanitize_text_field( $machine['name'] ) );
                update_post_meta( $order_id, '_shipping_address_2', sanitize_text_field( $machine['address'] ) );
                update_post_meta( $order_id, '_shipping_city', sanitize_text_field( $machine['city'] ) );
                update_post_meta( $order_id, '_shipping_postcode', '' );
                update_post_meta( $order_id, '_parcel_machine', sanitize_text_field( $_POST[$this->id] ) );
            }
        }
    }

    /**
     * Convert old methods to shipping zones if current site wc_mc_version is older than one of this module
     * 
     * @since 3.0.0
     */
    private function check_updates() {

        if ( get_site_option('wc_mc_version', 0) < WC_MC_VERSION) {

            update_site_option('wc_mc_version', WC_MC_VERSION);

            error_log("Need to update from [" . get_site_option( 'wc_mc_version', 0 ) . "] to [" . WC_MC_VERSION . "]");

            if ( WC_MC_VERSION === 2.0 ) {

                // Check if omniva / smartpost is enabled and in which countries
                // Convert to shipping zone
                foreach ( array(
                    'parcelmachine_omniva' => 'MakeCommerce\Shipping\Method\ParcelMachine\Omniva', 
                    'parcelmachine_smartpost' => 'MakeCommerce\Shipping\Method\ParcelMachine\Smartpost', 
                    'parcelmachine_dpd' => 'MakeCommerce\Shipping\Method\ParcelMachine\DPD'
                    ) as $method_name => $method_class ) {

                    $parcel_machine_method = new $method_class();

                    if ( $parcel_machine_method->enable === 'yes' ) {

                        $zones = \WC_Shipping_Zones::get_zones();

                        //skip if there were no countries selected
                        if ( !empty( $parcel_machine_method->countries ) ) {
                            continue;
                        }

                        foreach ( $parcel_machine_method->countries as $ecountry ) {

                            $has_ecountry = false;

                            foreach ( $zones as $zone ) {
                                foreach ( $zone['zone_locations'] as $location ) {

                                    if ( $location->code === $ecountry ) {

                                        $has_ecountry = $zone['zone_id'];

                                        //skip complete class if a shippingzone with this name has alread been defined
                                        foreach ( $zone['shipping_methods'] as $method ) {
                                            if ( $method->id === $method_name ) {
                                                
                                                continue 4;
                                            }
                                        }
                                    }
                                }
                            }

                            //convert to shipping zones
                            if ( !$has_ecountry ) {

                                $zone = new \WC_Shipping_Zone();
                                $zone->set_zone_name( $ecountry );
                                $zone->add_location( $ecountry, 'country' );

                                $zone->save();
                            } else {
                                $zone = new \WC_Shipping_Zone( $has_ecountry );
                            }

                            $instance_id = $zone->add_shipping_method($method_name);
                            $transport_class = new $method_class($instance_id);

                            $price = !empty( $transport_class->settings['price_'.strtolower($ecountry)] ) ? $transport_class->settings['price_'.strtolower($ecountry)] : 0;

                            $free_shipping_min_amount = !empty( $transport_class->settings['free_shipping_min_amount_'.strtolower( $ecountry )] ) ? $transport_class->settings['free_shipping_min_amount_'.strtolower($ecountry)] : 0;
                            $transport_class->set_post_data( array( $transport_class->get_field_key( 'price' ) => $price, $transport_class->get_field_key( 'free_shipping_min_amount' ) => $free_shipping_min_amount ) );

                            $transport_class->process_admin_options();
                        }
                    }
                }
            }
        }
    }

    /**
     * Called by shipping_ui_javascript (form fields)
     * 
     * @since 3.0.0
     */
    public function generate_shipping_ui_javascript_html( $key, $data ) {

        echo '
        <script type="text/javascript">
        jQuery(document).ready(function($) {

            var country_select = $("#woocommerce_parcelmachine_' . strtolower( $this->ext ) . '_countries");
            var prices = [], free_shippings = [];
            
            $.each(country_select.find("option"), function(i, option) {
                prices[$(option).val()] = $("#woocommerce_parcelmachine_' . strtolower( $this->ext ) . '_price_" + $(option).val().toLowerCase()).closest("tr");
                free_shippings[$(option).val()] = $("#woocommerce_parcelmachine_' . strtolower( $this->ext ) . '_free_shipping_min_amount_" + $(option).val().toLowerCase()).closest("tr");
            });

            country_select.on("change", function(){
                var countries = $(this).val();
                if(countries) {
                    hidePrices(countries);
                }
            });

            hidePrices(country_select.val());
            
            function hidePrices(countries) {
                for (var key in prices) {
                    if(countries.indexOf(key) == -1){
                        prices[key].hide();
                        free_shippings[key].hide();
                    } else {
                        prices[key].show();
                        free_shippings[key].show();
                    }
                }
            }
        });
        </script>
        ';
    }

    /**
     * Checks if there is a product that doesn't fit in parcelmachine
     * 
     * @since 3.0.0
     */
    private function fits_parcel_machine( $package ) {

        foreach ( $package['contents'] as $line ) {

            if ( get_post_meta( $line['product_id'], '_no_parcel_machine', true ) === 'yes' ) {
                return false;
            }
        }

        return true;
    }

    /**
     * Checks if shipping method is available.
     * In addition to parent is_available, also checks if package fits in parcel machine
     * 
     * @since 3.0.0
     */
    public function is_available( $package ) {

        if ( !$this->fits_parcel_machine( $package ) ) {
            return false;
        }

        return parent::is_available( $package );
    }

    /**
     * Sort machines (biggest cities first)
     * 
     * since 3.0.0
     */
    private function sort_machines( $machines ) {

        if ( $this->prioritization === 'yes' ) {
            
            usort( $machines, function( $a, $b ) {

                $sortorder = array(
                    'tallinn', 'tartu linn','tartu', 'narva', 'pärnu linn','pärnu', 'viljandi', 'kohtla-järve', 'rakvere', 'maardu', 'sillamäe', 'kuressaare',
                    'helsinki', 'espoo', 'tampere', 'vantaa', 'oulu', 'turku', 'jüväskülä', 'lahti', 'kuopio', 'kouvola',
                    'rīga','riga', 'daugavpils', 'liepāja','liepaja', 'jelgava', 'jūrmala','jurmala', 'ventspils', 'rezekne', 'valmiera', 'jekabpils',
                    'vilnius', 'kaunas', 'klaipeda', 'siauliai', 'panevezys', 'alytus', 'mariampole', 'mazeikiai', 'jonava', 'utena'
                );

                $acity = mb_strtolower($a['city']);
                $bcity = mb_strtolower($b['city']);
                
                if ( !$acity ) {
                    $acity = 'xxxxxxx';
                }

                if ( !$bcity ) {
                    $bcity = 'xxxxxxx';
                }

                $aidx = array_search( $acity, $sortorder );
                $bidx = array_search ($bcity, $sortorder );

                if ( $aidx !== false ) {
                    $acity = str_pad( $aidx, 4, "0", STR_PAD_LEFT ) . '-' . $acity;
                }

                if ( $bidx !== false ) {
                    $bcity = str_pad( $bidx, 4, "0", STR_PAD_LEFT ) . '-' . $bcity;
                }

                $acity .= '-' . mb_strtolower($a['name']);
                $bcity .= '-' . mb_strtolower($b['name']);
                
                return $acity < $bcity ? -1 : 1;
            } );
        }

        return $machines;
    }
    
    /**
     * Check if provided phonenumber is valid for parcelmachines
     * 
     * This function can/should be improved
     * 
     * @since 3.0.0
     */
    public function valid_phone_number( $phone ) {

        //remove spaces and remove + from phonenumber
        $phone = trim($phone);
        $phone = trim($phone, '+');

        //if phonenumber is starts with 3725 or 5 deem it valid.
        if ( substr( $phone, 0, 4 ) === '3725' || substr( $phone, 0, 1 ) === '5' ) {
            return true;
        }

        //Run through the list of method allowed country codes
        foreach ( $this->valid_phonenumber_country_codes as $country_code ) {

            if ( substr( $phone, 0, 3 ) === $country_code ) {
                return true;
            }
        }

        return false;
    }

    /**
     * Force creating a function for initializing method specific form fields
     * 
     * @since 3.0.0
     */
    abstract public function initialize_method_form_fields();
}